package com.buglabs.dragonfly.cdt.poky.ui.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import com.buglabs.dragonfly.cdt.poky.ui.PokyActivator;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UserInfo;

/**
 * A utility class that aids is copying local file to a remote host
 * @author akravets
 *
 */
public class SCP {
	private String localFile;
	private String remoteFilePath;
	private String user;
	private String hostName;
	private String password;
	private Session session;

	/**
	 * Constructor
	 * @param localFile Full path to the file that needs to be copied
	 * @param dir Directory on the server where file should be copied to
	 * @param user user name
	 * @param hostName host name of the server to copy file to
	 * @param password super user password for local machine
	 * @throws JSchException
	 * @throws IOException
	 */
	public SCP(String localFile, String dir, String user, String hostName, String password) throws JSchException, IOException {
		this.localFile = localFile;
		this.remoteFilePath = dir + File.separator + localFile.substring(localFile.lastIndexOf(File.separator) + 1, localFile.length());
		this.user = user;
		this.hostName = hostName;
		this.password = password;
	}

	/**
	 * Copies file to remote host
	 * @return <code>true</code> if copy succeeded, <code>false</code> otherwise. 
	 */
	public boolean copyFile(){
		String fileName = localFile.substring(localFile.lastIndexOf(File.separator) + 1, localFile.length());
		try {
			connectToHost();
			copy();
		} catch (JSchException e) {
			UIUtils.handleVisualError("Unable to copy " + fileName, e);
			return false;
		} catch (IOException e) {
			UIUtils.handleVisualError("Unable to copy " + fileName, e);
			return false;
		}
		return true;
	}

	private void connectToHost() throws JSchException {
		JSch jsch = new JSch();
		
		jsch.setKnownHosts(System.getProperty("user.home") + File.separator + ".ssh" + File.separator + "known_hosts");
		
		session = jsch.getSession(user, hostName, 22);
		UserInfo ui = new UserInfoImpl(password);
		session.setUserInfo(ui);

		Properties config = new Properties();
		config.put("StrictHostKeyChecking", "no"); // set property so that new hosts key are added to user known hosts
		session.setConfig(config);

		session.connect();
	}

	private void copy() throws JSchException, IOException{
		// exec 'scp -t rfile' remotely
		String command="scp -p -t "+remoteFilePath;
		Channel channel=session.openChannel("exec");
		((ChannelExec)channel).setCommand(command);

		OutputStream out = null;
		InputStream in = null;
		FileInputStream fis = null;

		try{
			// get I/O streams for remote scp
			out = channel.getOutputStream();
			in = channel.getInputStream();

			channel.connect();

			if(checkAck(in) != 0){
				System.out.println("check fail");
			}

			// send "C0644 filesize filename", where filename should not include '/'
			long filesize=(new File(localFile)).length();

			command="C0555 " + filesize + " ";

			if(localFile.lastIndexOf('/') > 0){
				command+=localFile.substring(localFile.lastIndexOf('/') + 1);
			}
			else{
				command += localFile;
			}

			command += "\n";

			out.write(command.getBytes()); out.flush();

			if(checkAck(in) != 0){
				System.out.println("check fail");
			}

			// send a content of localFile
			fis = new FileInputStream(localFile);
			byte[] buf = new byte[1024];

			while(true){
				int len = fis.read(buf, 0, buf.length);
				if(len <= 0) break;
				out.write(buf, 0, len); //out.flush();
			}

			// send '\0'
			buf[0] = 0; 
			out.write(buf, 0, 1); out.flush();

			if(checkAck(in) != 0){
				System.out.println("check fail");
			}

			channel.disconnect();
			session.disconnect();
		}
		finally{
			if(in != null || out != null || fis != null){
				in.close();
				out.close();
				fis.close();
			}
		}
	}

	private int checkAck(InputStream in) throws IOException{
		int b=in.read();
		// b may be 0 for success,
		// 1 for error,
		// 2 for fatal error,
		// -1
		if(b == 0) 
			return b;
		if(b == -1) 
			return b;

		if(b==1 || b==2){
			StringBuffer sb=new StringBuffer();
			int c;

			do {
				c=in.read();
				sb.append((char)c);
			}

			while(c!='\n');

			if(b == 1 || b == 2){ // error
				throw new IOException(sb.toString());
			}
		}
		return b;
	}

	private class UserInfoImpl implements UserInfo{

		private String password;

		public UserInfoImpl(String password) {
			this.password = password;
		}

		public String getPassphrase() {
			return password;
		}

		public String getPassword() {
			return password;
		}

		public boolean promptPassphrase(String arg0) {
			// TODO Auto-generated method stub
			return true;
		}

		public boolean promptPassword(String arg0) {
			// TODO Auto-generated method stub
			return true;
		}

		public boolean promptYesNo(String arg0) {
			// TODO Auto-generated method stub
			return false;
		}

		public void showMessage(String arg0) {
			// TODO Auto-generated method stub

		}
	}
}
